//
// Tiny OS with critical section and delays.
//

#include "lpc2478.h"
#include "init.h"
#include "print.h"
#include "os.h"

// Allocate stacks one for each task
int stackOne[STACKSIZE];
int stackTwo[STACKSIZE];
int stackThree[STACKSIZE];
int stackFour[STACKSIZE];
int stackFive[STACKSIZE];

void ShellTask(void);
void TaskOne(void);
void TaskTwo(void);
void TaskLedOn(void);
void TaskLedOff(void);

extern EventBlock* countSem;
extern BOOLEAN counting;

///////////////////////////////////////////////////////////////////////////////////////////
//
//  main
//
//  Creates three tasks with CreateTask which initialize their TaskBlocks and stacks.
//  Starts the timer for the interrupt ticks which cause the tasks to switch.
//  Calls StartOS with sets the current task and switches to it.
//
//  This function never runs passed the call to StartOS() so it never exits.
//
int main(void)
{
    InitHardware();

    // Clear screen and move cursor to the top
    printString("\033[2J\033[f");

    printF("University of Washington - TinyOS Shell\n");

    OSInit();

    counting = 0;
    countSem = OSCreateSemaphore(0);

    // Create tasks
    OSCreateTask(1, stackOne, (void*)ShellTask);
    OSCreateTask(2, stackTwo, (void*)TaskOne);
    OSCreateTask(3, stackThree, (void*)TaskTwo);
    OSCreateTask(4, stackFour, (void*)TaskLedOn);
    OSCreateTask(5, stackFive, (void*)TaskLedOff);

    InitializeTimerTick();

    OSStart();

    // Never gets here.
    printString("Main completed.\n");
    while(1);
    return 0;
}


