/*
 * MODULE
 *
 *   print.c
 *
 * DESCRIPTION
 *
 * Very lightweight print functions.
 *
****************************************************************************/
#define __PRINT_C__

#include "lpc2478.h"
#include "print.h"
#include <stdarg.h>

// Windows line endings: CR+LF ('\r\n', 0x0D 0x0A)

#define PUTC(c)  while(!(U0LSR & ULSR_THRE)) continue; \
	             U0THR = (uint8_t) c

#define GETC()  'x'


void printHex(Int32U u32)
{
    Int32U   u32Mask  = 0xF0000000;
    Int8U    u32Shift = 32;
    Int32U   u32Char;

    do {
        u32Shift -= 4;
        u32Char = (u32 & u32Mask) >> u32Shift;
        u32Mask >>= 4;
        if (u32Char >= 0xA)
        {
            PUTC('A' + (u32Char - 10));
        }
        else
        {
            PUTC('0' + u32Char);
        }
    } while (u32Shift > 0);
}

void printUint32(Int32U u)
{
    Int32U    buffer_length = 20;
    char      buffer[buffer_length];
    char      *p = &buffer[buffer_length - 1];

    *p = '\0';
    do {
        p--;
        *p = (u % 10) + '0';
        u /= 10;
    } while (u > 0);

    printString(p);
}

void printStringHex(char *ptr, Int32U u32Hex)
{
    printString(ptr);
    printHex(u32Hex);
    printChar('\n');
}

void printChar(const char letter)
{
    if (0 == letter)
    {
        return;
    }
    if ('\n' == letter)
    {
    	PUTC('\r');
    	PUTC('\n');
        return;
    }
    PUTC(letter);
    return;
}

void printString(char *ptr)
{
    if (ptr==0 || *ptr==0) return;

    do
    {
      if (*ptr=='\n')
      {
    	  PUTC('\r');
    	  PUTC(*ptr++);
      }
      else
      {
    	  PUTC(*ptr++);
      }
    }
    while (*ptr!=0);
}

void printF(char *format, ...)
{
    va_list args;
    va_start(args, format);

    char* ptr = format;

    while(*ptr != 0)
    {
    	switch(*ptr)
    	{
			case '\n':
				PUTC('\r');
				PUTC(*ptr++);
				break;
			case '%':
				ptr++;
				if (*ptr != 0)
				{
					switch(*ptr)
					{
//						case 'c':
//							printChar(va_arg(args, char));
//							break;
						case 's':
							printString(va_arg(args, char*));
							break;
						case 'd':
							printUint32(va_arg(args, unsigned int));
							break;
						case 'x':
							printHex(va_arg(args, unsigned int));
							break;
						default:
							// ignore unknown specifier
							break;
					}
					ptr++;
				}
				break;
			default:
				PUTC(*ptr++);
				break;
    	}
    }

    va_end(args);
}

