#include "lpc2478.h"
#include "print.h"
#include "os.h"

// Share semaphore created in main.c
EventBlock* countSem;

int taskOneCount;
int taskTwoCount;
BOOLEAN counting;

///////////////////////////////////////////////////////////////////////////////////////////
//
// TaskOne counts up from 0  and prints 10 lines.
//
void TaskOne(void)
{
	taskOneCount = 0;
	while(1)
	{
		// Wrap this for loop with the semaphore
		// to serialize the output

		OSPendSemaphore(countSem);

		for (int c=0; c<10; c++)
		{
			taskOneCount++;
			for(int i=0; i<3000;i++);
		}

		OSPostSemaphore(countSem);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////
//
// TaskTwo counts down from 0xFFFFFFFF and prints 10 lines.
//
void TaskTwo(void)
{
	taskTwoCount = 0xFFFFFFFF;
	while(1)
	{
		// Wrap this for loop with the semaphore
		// to serialize the output

		OSPendSemaphore(countSem);

		for (int c=0; c<10; c++)
		{
			taskTwoCount--;
			for(int i=0; i<3000;i++);
		}

		OSPostSemaphore(countSem);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////
//
// Function to turn LED on or off
//
static void SetLED(int On)
{
    if (On)
    {
    	FIO1SET_bit.P1_5 = 1;
    }
    else
    {
    	FIO1CLR_bit.P1_5 = 1;
    }
}

///////////////////////////////////////////////////////////////////////////////////////////
//
// Turn LED on task
//
void TaskLedOn(void)
{
	while(1)
	{
		SetLED(1);
		OSDelayTask(6);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////
//
// Turn LED off task
//
void TaskLedOff(void)
{
	OSDelayTask(3);
	while(1)
	{
		SetLED(0);
		OSDelayTask(6);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////
//
// Start TaskOne and TaskTwo counting by posting to their shared semaphore.
//
int StartCount(int argc, char **argv)
{
	if (!counting)
	{
		OSPostSemaphore(countSem);
		counting = 1;
		printF("Count started.\n");
	}
	else
	{
		printF("Count is already running.\n");
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////
//
// Stops tasks from counting by taking their semaphore.
//
int StopCount(int argc, char **argv)
{
	if (counting)
	{
		OSPendSemaphore(countSem);
		counting = 0;
		printF("Count stopped\n");
	}
	else
	{
		printF("Count is not running.\n");
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////
//
// Prints the current count.
//
int ShowCount(int argc, char **argv)
{
	printF("Count is %s.\n", counting ? "running" : "stopped");
	printF("TaskOne count: %d\n", taskOneCount);
	printF("TaskTwo count: %d\n", taskTwoCount);
	return 0;
}



